/*****************************************************************************
 * Copyright 2012 Zdenko Vrabel
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package org.zdevra.sparrow;

/**
 * All classes which is implementing this interface are somehow reacting on sparrow events like starting, shutdown etc.
 *
 * @author Zdenko Vrabel (vrabel.zdenko@gmail.com)
 */
public interface SparrowLifecycle
{
	/**
	 * This method is invoked before start of units and you should place here initialization code.
	 */
	public void onInit(SparrowEngine engine);

	/**
	 * This method is invoked when {@link SparrowEngine engine} starts all parts
	 */
	public void onStart();

	/**
	 * This method is invoked when  {@link SparrowEngine engine} stop all his parts
	 */
	public void onStop() throws InterruptedException;
}
