/*****************************************************************************
 * Copyright 2012 Zdenko Vrabel
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package org.zdevra.sparrow;

import java.util.concurrent.ThreadFactory;

/**
 * All Sparrow's threads are constructed via this factory impl.
 * @author Zdenko Vrabel (vrabel.zdenko@gmail.com)
 */
public class SparrowThreadFactory implements ThreadFactory
{
	//-------------------------------------------------------------------------------------------------------------------
	// members
	//-------------------------------------------------------------------------------------------------------------------

	private static SparrowThreadFactory singletonedThreadFactory;
	private final ThreadGroup sparrowThreadGroup;
	private volatile String threadName;

	//-------------------------------------------------------------------------------------------------------------------
	// static methods
	//-------------------------------------------------------------------------------------------------------------------

	public static synchronized Thread createThread(Runnable r, String name)
	{
		if (singletonedThreadFactory == null) {
			singletonedThreadFactory = new SparrowThreadFactory();
		}

		singletonedThreadFactory.threadName = name;
		return singletonedThreadFactory.newThread(r);
	}

	//-------------------------------------------------------------------------------------------------------------------
	// constructor
	//-------------------------------------------------------------------------------------------------------------------

	private SparrowThreadFactory()
	{
		sparrowThreadGroup = new ThreadGroup("sparrow-group");
	}

	//-------------------------------------------------------------------------------------------------------------------
	// impl. methods
	//-------------------------------------------------------------------------------------------------------------------

	@Override
	public Thread newThread(Runnable r)
	{
		return new Thread(this.sparrowThreadGroup, r, "sparrow-thread-" + this.threadName);
	}
}
