/*****************************************************************************
 * Copyright 2012 Zdenko Vrabel
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package org.zdevra.sparrow.actor;

import org.zdevra.sparrow.channel.ChannelReference;
import org.zdevra.sparrow.mailbox.Mailbox;

import java.util.UUID;

/**
 * Abstract builder with common behaviour used by both standalone and guice engines.
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public abstract class AbstractActorBuilder implements ActorBuilder
{
	//-------------------------------------------------------------------------------------------------------------------
	// members
	//-------------------------------------------------------------------------------------------------------------------

	/** name of input channel */
	protected String actorName;

	/** name of input channel */
	protected String inputChannel;

	/** name of output channel */
	protected String outputChannel;

	/** mailbox implementation */
	protected Mailbox mailbox;

	/** handler for all uncaught exceptions */
	protected ActorExceptionHandler exceptionHandler;

	/** builder for actor handler */
	protected ActorHandlerBuilder handlerBuilder;

	//-------------------------------------------------------------------------------------------------------------------
	// constructor
	//-------------------------------------------------------------------------------------------------------------------

	/**
	 * Constructor
	 *
	 * @param handlerBuilder
	 *
	 */
	protected AbstractActorBuilder(ActorHandlerBuilder handlerBuilder)
	{
		this.handlerBuilder = handlerBuilder;
		this.actorName = UUID.randomUUID().toString();
	}


	//-------------------------------------------------------------------------------------------------------------------
	// methods
	//-------------------------------------------------------------------------------------------------------------------

	@Override
	public final ActorBuilder namedAs(String name)
	{
		this.actorName = name;
		return this;
	}


	@Override
	public final ActorBuilder mailbox(Mailbox m)
	{
		this.mailbox = m;
		return this;
	}


	@Override
	public final ActorBuilder exceptionsAreHandled(ActorExceptionHandler handler)
	{
		this.exceptionHandler = handler;
		return this;
	}


	@Override
	public String name()
	{
		return actorName;
	}


	@Override
	public ActorBuilder to(ChannelReference output)
	{
		this.outputChannel = output.name();
		return this;
	}


	@Override
	public ActorBuilder from(ChannelReference input)
	{
		if (input != null) {
			this.inputChannel = input.name();
		}
		return this;
	}
}
