/*****************************************************************************
 * Copyright 2012 Zdenko Vrabel
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package org.zdevra.sparrow.actor;

import org.zdevra.sparrow.SparrowBuilder;
import org.zdevra.sparrow.channel.ChannelReference;
import org.zdevra.sparrow.mailbox.FifoMailbox;
import org.zdevra.sparrow.mailbox.Mailbox;

/**
 * Builder interface for {@link PublisherSubsriberActor actors}.
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public interface ActorBuilder extends ChannelReference, SparrowBuilder
{
	/**
	 * name for actor
	 *
	 * @param name
	 *           name of actor
	 *
	 * @return
	 */
	public ActorBuilder namedAs(String name);


	/**
	 * set output channel's name for actor.
	 *
	 * @param output
	 *           reference to output
	 * @return
	 */
	public ActorBuilder to(ChannelReference output);


	/**
	 * set input channel's name for actor.
	 *
	 * @param input
	 *           reference to input ch.
	 * @return
	 */
	public ActorBuilder from(ChannelReference input);



	/**
	 * I you want to use another mailbox, or mailbox with different capacity as default is. As default is used
	 * {@link FifoMailbox} with capacity 100.
	 *
	 * @param m
	 *        mailbox instance you want to use
	 *
	 * @return
	 */
	public ActorBuilder mailbox(Mailbox m);


	/**
	 * set the {@link ActorExceptionHandler handler} for actor which
	 * handles all uncaught exceptions
	 *
	 * @param handler
	 * @return
	 */
	public ActorBuilder exceptionsAreHandled(ActorExceptionHandler handler);

}
