/*****************************************************************************
 * Copyright 2012 Zdenko Vrabel
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package org.zdevra.sparrow.actor;

import org.zdevra.sparrow.channel.MessageChannel;

/**
 * This is base actor handler which provide you method send() that is sending the message into predefined
 * output channel. The output channel could be any {@link org.zdevra.sparrow.channel.MessageChannel channel}.
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public abstract class OutputActorHandler<T> implements ActorHandler<T>, ActorOutput
{
	//-------------------------------------------------------------------------------------------------------------------
	// members
	//-------------------------------------------------------------------------------------------------------------------

	/** output channel which is setup by sparrow (mostly injected from module) where send() sends the message */
	private MessageChannel output;

	//-------------------------------------------------------------------------------------------------------------------
	// methods
	//-------------------------------------------------------------------------------------------------------------------


	/**
	 * returns output message that you could use for sending messages
	 * from your actor.
	 */
	@Override
	public final MessageChannel output()
	{
		return output;
	}


	/**
	 * {@inheritDoc}
	 */
	@Override
	public final void setOutput(MessageChannel output)
	{
		this.output = output;
	}
}
