package org.zdevra.sparrow.actor;

import org.zdevra.sparrow.channel.ChannelReference;
import org.zdevra.sparrow.channel.NamedChannelReference;
import org.zdevra.sparrow.SparrowEngine;
import org.zdevra.sparrow.channel.MessageChannel;
import org.zdevra.sparrow.channel.SubscribableChannel;
import org.zdevra.sparrow.mailbox.Mailbox;

/**
 * Actor extended about input and output channels that means actor could be used as subscriber for
 * subscribable {@link SubscribableChannel channels} and also you could send message to output channel via output()
 * method.
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public abstract class PublisherSubsriberActor extends SubscriberActor implements ActorOutput
{
	//-------------------------------------------------------------------------------------------------------------------
	// members
	//-------------------------------------------------------------------------------------------------------------------

	/** reference of output channel that will be used */
	private volatile ChannelReference outputRef;

	/** output channel that will be used */
	private volatile MessageChannel output;


	//-------------------------------------------------------------------------------------------------------------------
	// constructor
	//-------------------------------------------------------------------------------------------------------------------

	/**
	 * Constructor
	 *
	 * @param mailbox
	 * @param exceptionHandler
	 * @param inputName
	 * @param outputName
	 */
	protected PublisherSubsriberActor(
			  Mailbox mailbox,
			  ActorExceptionHandler exceptionHandler,
			  String inputName,
			  String outputName)
	{
		super(mailbox, exceptionHandler, inputName);
		this.outputRef = new NamedChannelReference(outputName);
		this.output = null;
	}


	/**
	 * Constructor
	 *
	 * @param mailbox
	 * @param exceptionHandler
	 * @param input
	 * @param output
	 */
	protected PublisherSubsriberActor(
			  Mailbox mailbox,
			  ActorExceptionHandler exceptionHandler,
			  SubscribableChannel input,
			  MessageChannel output)
	{
		super(mailbox, exceptionHandler, input);
		this.output = output;
	}


	//-------------------------------------------------------------------------------------------------------------------
	// impl. methods
	//-------------------------------------------------------------------------------------------------------------------


	/**
	 * {@inheritDoc}
	 */
	@Override
	public void onInit(SparrowEngine engine)
	{
		if (output == null) {
			output = engine.getMessageChannel(outputRef);
		}
		super.onInit(engine);
	}


	/**
	 * {@inheritDoc}
	 */
	@Override
	public final void setOutput(MessageChannel output)
	{
		this.output = output;
	}


	/**
	 * {@inheritDoc}
	 */
	@Override
	public final MessageChannel output()
	{
		return output;
	}
}
