package org.zdevra.sparrow.actor;

import org.zdevra.sparrow.message.Message;
import org.zdevra.sparrow.SparrowEngine;
import org.zdevra.sparrow.channel.MessageChannel;
import org.zdevra.sparrow.channel.SubscribableChannel;
import org.zdevra.sparrow.mailbox.Mailbox;

/**
 * Standalone actor implementation with input/output channels that exclude processing into actor's
 * {@link ActorHandler handler}.
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public class StandaloneActor extends PublisherSubsriberActor
{
	//-------------------------------------------------------------------------------------------------------------------
	// members
	//-------------------------------------------------------------------------------------------------------------------

	/** actor handler which implements the actor's procedure */
	protected volatile ActorHandler<?> actorHandler;


	//-------------------------------------------------------------------------------------------------------------------
	// constructors
	//-------------------------------------------------------------------------------------------------------------------

	/**
	 * Constructor
	 *
	 * @param handler
	 * @param mailbox
	 * @param exceptionHandler
	 * @param inputName
	 * @param outputName
	 */
	public StandaloneActor(ActorHandler<?> handler, Mailbox mailbox, ActorExceptionHandler exceptionHandler, String inputName, String outputName)
	{
		super(mailbox, exceptionHandler, inputName, outputName);
		this.actorHandler = handler;
	}


	/**
	 * Constructor
	 *
	 * @param handler
	 * @param mailbox
	 * @param exceptionHandler
	 * @param input
	 * @param output
	 */
	public StandaloneActor(ActorHandler<?> handler, Mailbox mailbox, ActorExceptionHandler exceptionHandler, SubscribableChannel input, MessageChannel output)
	{
		super(mailbox, exceptionHandler, input, output);
		this.actorHandler = handler;
	}

	//-------------------------------------------------------------------------------------------------------------------
	// methods
	//-------------------------------------------------------------------------------------------------------------------


	@Override
	public void onInit(SparrowEngine engine)
	{
		super.onInit(engine);
		if (actorHandler instanceof ActorOutput) {
			((ActorOutput)actorHandler).setOutput( output() );
		}
	}


	@Override
	protected final void onReceive(Message<?> msg)
	{
		actorHandler.act((Message)msg);
	}
}
