package org.zdevra.sparrow.actor;

import org.zdevra.sparrow.channel.ChannelReference;
import org.zdevra.sparrow.channel.NamedChannelReference;
import org.zdevra.sparrow.message.Message;
import org.zdevra.sparrow.SparrowEngine;
import org.zdevra.sparrow.channel.MessageHandler;
import org.zdevra.sparrow.channel.SubscribableChannel;
import org.zdevra.sparrow.mailbox.Mailbox;

/**
 * Actor extended about input channel that means actor could be used as subscriber for
 * subscribable {@link SubscribableChannel channels}
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public abstract class SubscriberActor extends AbstractActor implements MessageHandler
{
	//-------------------------------------------------------------------------------------------------------------------
	// members
	//-------------------------------------------------------------------------------------------------------------------

	/** reference to the input channel */
	private volatile ChannelReference inputRef;

	/** input channel for which the actor will be registered as subscriber */
	private volatile SubscribableChannel input;


	//-------------------------------------------------------------------------------------------------------------------
	// constructors
	//-------------------------------------------------------------------------------------------------------------------

	/**
	 * Constructor
	 *
	 * @param mailbox
	 *
	 * @param exceptionHandler
	 *
	 * @param input
	 *
	 */
	protected SubscriberActor(Mailbox mailbox, ActorExceptionHandler exceptionHandler, SubscribableChannel input)
	{
		super(mailbox, exceptionHandler);
		this.input = input;
	}


	/**
	 * Constructor
	 *
	 * @param mailbox
	 *
	 * @param exceptionHandler
	 *
	 * @param inputName
	 *
	 */
	protected SubscriberActor(Mailbox mailbox, ActorExceptionHandler exceptionHandler, String inputName)
	{
		super(mailbox, exceptionHandler);
		this.inputRef = new NamedChannelReference(inputName);
		this.input = null;
	}


	//-------------------------------------------------------------------------------------------------------------------
	// methods
	//-------------------------------------------------------------------------------------------------------------------


	@Override
	public final boolean handleMessage(Message<?> message)
	{
		return send(message);
	}


	/**
	 * {@inheritDoc}
	 */
	@Override
	public void onInit(SparrowEngine engine)
	{
		if (input == null) {
			input = (SubscribableChannel) engine.getMessageChannel(inputRef);
		}
		input.subscribe(this);
	}
}
