/*****************************************************************************
 * Copyright 2012 Zdenko Vrabel
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package org.zdevra.sparrow.adapter;

import org.zdevra.sparrow.*;
import org.zdevra.sparrow.channel.ChannelReference;
import org.zdevra.sparrow.channel.MessageChannel;
import org.zdevra.sparrow.channel.NamedChannelReference;
import org.zdevra.sparrow.message.MessageProducer;

import java.util.UUID;

/**
 * You will extend this class for your own inbound adapters that produce
 * messages.
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public abstract class InboundAdapter implements MessageProducer, SparrowLifecycle, Runnable
{
	//-------------------------------------------------------------------------------------------------------------------
	// members
	//-------------------------------------------------------------------------------------------------------------------

	/** channel where you will push messages */
	private ChannelReference outputRef;

	/** channel where you will push messages */
	private MessageChannel outputChannel;

	/** adapter's thread */
	private Thread adapterThread;


	//-------------------------------------------------------------------------------------------------------------------
	// methods for adapter impl.
	//-------------------------------------------------------------------------------------------------------------------


	/**
	 * Returns output channel which is bind to adapter
	 */
	protected MessageChannel output()
	{
		return outputChannel;
	}


	//-------------------------------------------------------------------------------------------------------------------
	// methods
	//-------------------------------------------------------------------------------------------------------------------

	/**
	 * Method setup name of output channel. Method serve for internal purposes and it's
	 * invoked by the AdapterBuilder.
	 *
	 * @param nameOfOutputChannel
	 */
	public final void setOutputName(String nameOfOutputChannel)
	{
		this.outputRef = new NamedChannelReference(nameOfOutputChannel);
	}


	/**
	 * {@inheritDoc}
	 */
	@Override
	public final void setOutputChannel(MessageChannel channel)
	{
		this.outputChannel = channel;
	}


	/**
	 * {@inheritDoc}
	 */
	@Override
	public final void onInit(SparrowEngine engine)
	{
		if (outputChannel == null) {
			outputChannel = engine.getMessageChannel(outputRef);
			setOutputChannel(outputChannel);
		}
	}


	/**
	 * {@inheritDoc}
	 */
	@Override
	public final void onStart()
	{
		adapterThread = SparrowThreadFactory.createThread(this, UUID.randomUUID().toString());
		adapterThread.setPriority(Thread.MIN_PRIORITY);
		adapterThread.start();
	}


	/**
	 * {@inheritDoc}
	 */
	@Override
	public final void onStop() throws InterruptedException
	{
		if (!adapterThread.isInterrupted()) {
			adapterThread.interrupt();
		}
	}

}
