package org.zdevra.sparrow.aggregator;

import org.zdevra.sparrow.actor.ActorHandler;

/**
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public class AggregatorBuilderImpl implements AggregatorBuilder
{
	//-------------------------------------------------------------------------------------------------------------------
	// members
	//-------------------------------------------------------------------------------------------------------------------

	private long timeout;

	private ReleaseStrategy releasingStrategy;

	private AggregatorStrategy aggregationStrategy;

	//-------------------------------------------------------------------------------------------------------------------
	// Constructors
	//-------------------------------------------------------------------------------------------------------------------

	/**
	 * Constructor
	 */
	public AggregatorBuilderImpl()
	{
		this.timeout = 1000;
		this.releasingStrategy = null;
		this.aggregationStrategy = null;
	}

	//-------------------------------------------------------------------------------------------------------------------
	// methods
	//-------------------------------------------------------------------------------------------------------------------

	@Override
	public AggregatorBuilder timeout(long timeout)
	{
		this.timeout = timeout;
		return this;
	}


	@Override
	public AggregatorBuilder withReleaseStrategy(ReleaseStrategy releasingStrategy)
	{
		this.releasingStrategy = releasingStrategy;
		return this;
	}


	@Override
	public AggregatorBuilder ofInstance(AggregatorStrategy aggregationStrategy)
	{
		this.aggregationStrategy = aggregationStrategy;
		return this;
	}


	@Override
	public AggregatorBuilder of(Class<?> aggregatorClass)
	{
		try
		{
			if (AggregatorStrategy.class.isAssignableFrom(aggregatorClass)) {
				this.aggregationStrategy = (AggregatorStrategy) aggregatorClass.newInstance();
			}
		} catch (InstantiationException e) {
			//todo: handle exception somehow
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			//todo: handle exception somehow
			e.printStackTrace();
		}

		return this;
	}


	@Override
	public ActorHandler<?> build()
	{
		return new AggregatorActorHandler(aggregationStrategy, releasingStrategy, timeout);
	}
}
