package org.zdevra.sparrow.chain;

import org.zdevra.sparrow.SparrowBuilder;
import org.zdevra.sparrow.dsl.SparrowDsl;
import org.zdevra.sparrow.actor.ActorBuilder;
import org.zdevra.sparrow.channel.ChannelReference;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

/**
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public class ChainBuilder implements SparrowBuilder
{
	//-------------------------------------------------------------------------------------------------------------------
	// members
	//-------------------------------------------------------------------------------------------------------------------
	private ChannelReference input;
	private ChannelReference output;
	private List<ActorBuilder> actorBuilders;
	private SparrowDsl dsl;
	private String chainName;

	//-------------------------------------------------------------------------------------------------------------------
	// constructor & methods
	//-------------------------------------------------------------------------------------------------------------------

	public ChainBuilder(ChannelReference input, String chainName, SparrowDsl dsl)
	{
		this.actorBuilders = new LinkedList<ActorBuilder>();
		this.input = input;
		this.chainName = chainName;
		this.dsl = dsl;
	}


	public ChainBuilder pipeline(ActorBuilder... actors)
	{
		actorBuilders.addAll(Arrays.asList(actors));
		return this;
	}


	public ChannelReference to(ChannelReference channel)
	{
		this.output = channel;
		return channel;
	}


	@Override
	public void build()
	{
		boolean isLast = false;
		int size = actorBuilders.size();
		ChannelReference inputChannel = input;

		int i = 0;
		for (; i < (size - 1); ++i)
		{
			ActorBuilder builder = actorBuilders.get(i);
			builder.from(inputChannel);

			inputChannel = dsl.channel("chain-" + chainName + "-ch-" + i);
			builder.to(inputChannel);
		}

		//setup last actor in chain
		ActorBuilder lastBuilder = actorBuilders.get(i);
		lastBuilder
				  .from(inputChannel)
				  .to(output);
	}
}
