/*****************************************************************************
 * Copyright 2012 Zdenko Vrabel
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package org.zdevra.sparrow.channel;

/**
 * Basic abstraction handling common behaviour around parameters.
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public abstract class AbstractChannelBuilder implements ChannelBuilder
{
	//-------------------------------------------------------------------------------------------------------------------
	// members
	//-------------------------------------------------------------------------------------------------------------------

	/** name of the channel in engine/module's context */
	protected String channelName;

	/** instance of channel that will be used */
	protected MessageChannel channelInstance;

	//-------------------------------------------------------------------------------------------------------------------
	// methods
	//-------------------------------------------------------------------------------------------------------------------

	/**
	 * {@inheritDoc}
	 */
	@Override
	public final ChannelBuilder namedAs(String channelName)
	{
		this.channelName = channelName;
		return this;
	}


	@Override
	public final ChannelBuilder asInstance(MessageChannel channelInstance)
	{
		this.channelInstance = channelInstance;
		return this;
	}


	@Override
	public String name()
	{
		return channelName;
	}
}
