/*****************************************************************************
 * Copyright 2012 Zdenko Vrabel
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package org.zdevra.sparrow.channel;

/**
 * Builder for {@link MessageChannel channels}.
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public interface ChannelBuilder extends ChannelReference
{
	/**
	 * setup name for channel
	 *
	 * @param channelName
	 *           channel name in engine's context
	 * @return
	 */
	public ChannelBuilder namedAs(String channelName);


	/**
	 * Mark channel as exposed from engine. Normally in Guice, all channels are private. You have to expose channel
	 * if you want to access to channel from different module.
	 */
	public ChannelBuilder exposed();


	/**
	 * Implementation/type of channel which will be used.
	 *
	 * @param channelClass
	 *
	 * @return
	 */
	public ChannelBuilder asClass(Class<? extends MessageChannel> channelClass);


	/**
	 * Implementation/type of channel which will be used.
	 *
	 * @param channelInstance
	 * @return
	 */
	public ChannelBuilder asInstance(MessageChannel channelInstance);


	/**
	 * Building procedure
	 */
	public void build();

}
