/*****************************************************************************
 * Copyright 2012 Zdenko Vrabel
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package org.zdevra.sparrow.channel;

import org.zdevra.sparrow.message.Message;

/**
 * Channel send messages with the same type of data (payload). All other messages will be refused by channel.
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public class DataTypeChannel extends DirectChannel
{

	//-------------------------------------------------------------------------------------------------------------------
	// members
	//-------------------------------------------------------------------------------------------------------------------

	/** data type class which will be accepted by channel */
	private final Class<?> dataType;


	//-------------------------------------------------------------------------------------------------------------------
	// constructor & methods
	//-------------------------------------------------------------------------------------------------------------------

	/**
	 * Constructor
	 * @param dataType
	 */
	public DataTypeChannel(Class<?> dataType)
	{
		this.dataType = dataType;
	}

	/**
	 * Checking whether payload is instance of allowed data type. If not, the message will be refused.
	 * {@inheritDoc}
	 */
	@Override
	public boolean send(Message<?> message)
	{
		Object payload = message.getPayload();
		if (!dataType.isInstance(payload)) {
			return false;
		}
		return super.send(message);
	}


	/**
	 * Checking whether payload is instance of allowed data type. If not, the message will be refused.
	 * {@inheritDoc}
	 */
	@Override
	public boolean send(Message<?> message, long timeout)
	{
		Object payload = message.getPayload();
		if (!dataType.isInstance(payload)) {
			return false;
		}
		return super.send(message, timeout);
	}
}
