/*****************************************************************************
 * Copyright 2012 Zdenko Vrabel
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package org.zdevra.sparrow.channel;

import org.zdevra.sparrow.Nameable;
import org.zdevra.sparrow.message.Message;

import java.util.List;

/**
 * Implementation sends the message directly to all
 * his subscribers.
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public class DirectChannel extends PublishSubscribeChannel
{
	/**
	 * Dispatch message to all subscribers. If one or more subscribers don't receive
	 * message (send method returns false), the dispatch method itself return false.
	 *
	 * @param msg
	 *
	 * @param subscribers
	 *
	 * @return true when all subscribers receive the message.
	 */
	@Override
	protected boolean dispatch(Message<?> msg, List<MessageHandler> subscribers)
	{
		boolean failed = false;
		for (MessageHandler subscriber : subscribers) {
			boolean res = subscriber.handleMessage(msg);
			if (!res) {
				failed = true;
			}
		}
		return !failed;
	}


	/**
	 * returns type and name of channel
	 */
	@Override
	public String toString()
	{
		return "DirectChannel(" + getName() + ")";
	}
}
