/*****************************************************************************
 * Copyright 2012 Zdenko Vrabel
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package org.zdevra.sparrow.channel;

import org.zdevra.sparrow.message.Message;

import java.util.List;

/**
 * Implementation of load balanced dispatching for publish-subscribe channel.
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public class LoadBalancerChannel extends PublishSubscribeChannel
{
	//-------------------------------------------------------------------------------------------------------------------
	// members
	//-------------------------------------------------------------------------------------------------------------------

	/** index is changed with every new message and tells what subscriber receive the message */
	private volatile int index = 0;

	//-------------------------------------------------------------------------------------------------------------------
	// impl. method
	//-------------------------------------------------------------------------------------------------------------------

	/**
	 * Divide messages between subsribers.
	 *
	 * @param msg
	 *
	 * @param subscribers
	 *
	 * @return
	 */
	@Override
	protected boolean dispatch(Message<?> msg, List<MessageHandler> subscribers)
	{
		index = nextIndex(subscribers);
		MessageHandler subscriber = subscribers.get(index);
		if (subscriber == null) {
			return false;
		}

		return subscriber.handleMessage(msg);
	}


	private synchronized int nextIndex(List<MessageHandler> subscribers)
	{
		index++;
		if (index >= subscribers.size()) {
			index = 0;
		}
		return index;
	}
}
