/*****************************************************************************
 * Copyright 2012 Zdenko Vrabel
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package org.zdevra.sparrow.channel;

import org.zdevra.sparrow.message.Message;

/**
 * Base channel interface defining message sending.
 *
 * @author Zdenko Vrabel (vrabel.zdenko@gmail.com)
 */
public interface MessageChannel
{
	/**
	 * Send a {@link Message message} into this channel. If method cannot be sent for various problems (no
	 * space in queue, channel reject it etc) it will return false. Depends on implementation whether method
	 * is blocking or non-blocking.
	 *
	 * @param message
	 *          message you want to send
	 *
	 * @return true if message was sent successful, false if message was rejected by channel for some reason
	 *
	 */
	boolean send(Message<?> message);


	/**
	 * blocking version of send with timeout which sends message into channel. Method blocking the execution until
	 * timeout expire.
	 *
	 * @param message
	 *          message you want to send
	 *
	 * @return true if message was sent successful, false if message was rejected by channel for some reason
	 *
	 */
	boolean send(Message<?> message, long timeout);
}
