package org.zdevra.sparrow.dsl;

import org.zdevra.sparrow.actor.ActorBuilder;
import org.zdevra.sparrow.actor.ActorHandler;
import org.zdevra.sparrow.actor.ActorHandlerBuilder;
import org.zdevra.sparrow.chain.ChainBuilder;
import org.zdevra.sparrow.channel.ChannelReference;

/**
 * DSL builder goes after {@code from} method.
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public abstract class FromDsl
{
	/** holds input channel ref*/
	protected ChannelReference inputChannelRef;


	/**
	 * Set what input channel will be used
	 */
	public void setInputChannel(ChannelReference input)
	{
		this.inputChannelRef = input;
	}


	/**
	 * This method could be used as input for {@code actor} method:
	 *
	 * <pre>
	 *    from( channelRef("somechannel").actor( chain("name") ).to( channelRef("outputchannel") );
	 * </pre>
	 *
	 * @param name
	 * @return
	 */
	public abstract ChainBuilder chain(String name);


	/**
	 * Builds actor for {@link ActorHandler} instance
	 *
	 * @param actor
	 * @return
	 */
	public abstract ActorBuilder actor(ActorHandler<?> actor);


	/**
	 * Builds actor where input could be another DSL {@link ActorHandlerBuilder builder}.
	 *
	 * @param actor
	 * @return
	 */
	public abstract ActorBuilder actor(ActorHandlerBuilder actor);


	/**
	 * Builds actor for {@link ActorHandler} class
	 *
	 * @param actor
	 * @return
	 */
	public abstract ActorBuilder actor(Class<? extends ActorHandler> actor);
}
