/*****************************************************************************
 * Copyright 2012 Zdenko Vrabel
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package org.zdevra.sparrow.filter;

import org.zdevra.sparrow.DiscardStrategy;
import org.zdevra.sparrow.actor.ActorHandler;
import org.zdevra.sparrow.actor.ActorHandlerBuilder;

import java.util.LinkedList;
import java.util.List;

/**
 * Builds filter with {@link FilterRule rules}
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public class FilterBuilder implements ActorHandlerBuilder
{
	//-------------------------------------------------------------------------------------------------------------------
	// members
	//-------------------------------------------------------------------------------------------------------------------

	/** list of rules for filter */
	protected List<FilterRule> rules;

	/** procedure for discarded messages */
	protected DiscardStrategy discardStrategy;

	//-------------------------------------------------------------------------------------------------------------------
	// constructor
	//-------------------------------------------------------------------------------------------------------------------

	/**
	 * Constructor
	 */
	public FilterBuilder()
	{
		this.rules = new LinkedList<FilterRule>();
	}


	//-------------------------------------------------------------------------------------------------------------------
	// methods
	//-------------------------------------------------------------------------------------------------------------------

	/**
	 * Add rules for filter
	 *
	 * @param rules
	 *           list of rules
	 * @return
	 */
	public final FilterBuilder when( FilterRuleBuilder... rules )
	{
		for (int i = 0; i < rules.length; ++i) {
			this.rules.add( rules[i].build() );
		}
		return this;
	}


	/**
	 * strategy how discarded/rejected messages will be processed.
	 *
	 * @param discardStrategy
	 * @return
	 */
	public final FilterBuilder discard(DiscardStrategy discardStrategy)
	{
		this.discardStrategy = discardStrategy;
		return this;
	}


	/**
	 * {@inheritDoc}
	 */
	@Override
	public ActorHandler<?> build()
	{
		return new FilterActorHandler(rules, discardStrategy);
	}
}
