/*****************************************************************************
 * Copyright 2012 Zdenko Vrabel
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package org.zdevra.sparrow.filter;

/**
 * Builder of {@code header()} in {@link org.zdevra.sparrow.AbstractSparrowEngine}
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public class HeaderFilterRuleBuilder implements FilterRuleBuilder
{
	//-------------------------------------------------------------------------------------------------------------------
	// members
	//-------------------------------------------------------------------------------------------------------------------

	/** header's name */
	private final String headerKey;

	/** header's value we want to compare with */
	private Object headerValue;

	/** 'not' flag that reverse result */
	private boolean reverse = false;


	//-------------------------------------------------------------------------------------------------------------------
	// constructor & methods
	//-------------------------------------------------------------------------------------------------------------------

	/**
	 * Constructor
	 * @param headerKey
	 */
	public HeaderFilterRuleBuilder(String headerKey)
	{
		this.headerKey = headerKey;
	}


	/**
	 * Negate the filter condition
	 *
	 * @return
	 */
	public HeaderFilterRuleBuilder not() {
		this.reverse = true;
		return this;
	}


	/**
	 * to what value we want compare the header's value
	 * @param value
	 * @return
	 */
	public HeaderFilterRuleBuilder isEqualsTo(Object value)
	{
		this.headerValue = value;
		return this;
	}


	/**
	 * {@inheritDoc}
	 */
	@Override
	public FilterRule build()
	{
		FilterRule rule = new HeaderRule(headerKey, headerValue);
		if (reverse) {
			rule = new ReverseRule(rule);
		}
		return rule;
	}
}
