/*****************************************************************************
 * Copyright 2012 Zdenko Vrabel
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package org.zdevra.sparrow.filter;

import org.zdevra.sparrow.message.Message;

/**
 * Filter that compares header's value to some concrete value
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public class HeaderRule implements FilterRule
{
	//-------------------------------------------------------------------------------------------------------------------
	// members
	//-------------------------------------------------------------------------------------------------------------------

	/** name of header */
	private final String headerKey;

	/** value of header */
	private final Object equalsToValue;


	//-------------------------------------------------------------------------------------------------------------------
	// constructor & methods
	//-------------------------------------------------------------------------------------------------------------------

	/**
	 * Constructor
	 * @param headerKey
	 * @param equalsToValue
	 */
	public HeaderRule(String headerKey, Object equalsToValue)
	{
		this.headerKey = headerKey;
		this.equalsToValue = equalsToValue;
	}


	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean accept(Message<?> msg)
	{
		Object headerValue = msg.getHeader().get(headerKey);
		if (headerValue == null) {
			return false;
		}

		boolean isEquals = headerValue.equals(equalsToValue);
		if (!isEquals) {
			return false;
		}

		return true;
	}
}
