/*****************************************************************************
 * Copyright 2012 Zdenko Vrabel
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package org.zdevra.sparrow.filter;

/**
 * Builder for payload filter rules.
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public class PayloadFilterRuleBuilder implements FilterRuleBuilder
{
	//-------------------------------------------------------------------------------------------------------------------
	// members
	//-------------------------------------------------------------------------------------------------------------------

	private Object payloadObj;
	private Class<?> payloadClass;
	private boolean reverse = false;

	//-------------------------------------------------------------------------------------------------------------------
	// methods
	//-------------------------------------------------------------------------------------------------------------------


	public PayloadFilterRuleBuilder not() {
		reverse = true;
		return this;
	}


	public PayloadFilterRuleBuilder isEqualsTo(Object payloadObj) {
		this.payloadObj = payloadObj;
		return this;
	}


	public PayloadFilterRuleBuilder isTypeOf(Class<?> payloadClass) {
		this.payloadClass = payloadClass;
		return this;
	}

	//-------------------------------------------------------------------------------------------------------------------
	// build procedure
	//-------------------------------------------------------------------------------------------------------------------

	@Override
	public FilterRule build()
	{
		FilterRule rule = null;

		if (payloadObj != null) {
			rule = new PayloadRule(payloadObj);
		} else if (payloadClass != null) {
			rule = new PayloadTypeRule(payloadClass);
		}

		if (reverse) {
			rule = new ReverseRule(rule);
		}

		return rule;
	}
}
