/*****************************************************************************
 * Copyright 2012 Zdenko Vrabel
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package org.zdevra.sparrow.filter;

import org.zdevra.sparrow.message.Message;

/**
 * Filter compare message's payload and if it's equal, then
 * is accepted, otherwise the message is rejected.
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public class PayloadRule implements FilterRule
{
	//-------------------------------------------------------------------------------------------------------------------
	// members
	//-------------------------------------------------------------------------------------------------------------------

	/** payload will be compared with this value */
	private final Object equalsToValue;

	//-------------------------------------------------------------------------------------------------------------------
	// constructor & methods
	//-------------------------------------------------------------------------------------------------------------------

	/**
	 * Constructor
	 * @param equalsToValue
	 */
	public PayloadRule(Object equalsToValue)
	{
		this.equalsToValue = equalsToValue;
	}


	/**
	 * compare message's payload
	 * {@inheritDoc}
	 */
	@Override
	public boolean accept(Message<?> msg)
	{
		Object payload = msg.getPayload();
		if (equalsToValue.equals(equalsToValue)) {
			return true;
		}
		return false;
	}

}
