/*****************************************************************************
 * Copyright 2012 Zdenko Vrabel
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package org.zdevra.sparrow.filter;

import org.zdevra.sparrow.message.Message;

/**
 * Decorator rule which reverse result of decorated {@link FilterRule rule}. It's used as {@code not()} method.
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public class ReverseRule implements FilterRule
{
	//-------------------------------------------------------------------------------------------------------------------
	// members
	//-------------------------------------------------------------------------------------------------------------------

	/** filter rule that is decorated */
	private final FilterRule decoratedRule;

	//-------------------------------------------------------------------------------------------------------------------
	// constructor & methods
	//-------------------------------------------------------------------------------------------------------------------

	/**
	 * Constructor
	 * @param decoratedRule
	 */
	public ReverseRule(FilterRule decoratedRule)
	{
		this.decoratedRule = decoratedRule;
	}


	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean accept(Message<?> msg)
	{
		return decoratedRule.accept(msg);
	}
}
