/*****************************************************************************
 * Copyright 2012 Zdenko Vrabel
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package org.zdevra.sparrow.mailbox;

import org.zdevra.sparrow.message.Message;

/**
 * FIFO blocking queue implementation of mailbox. This mailbox change the InputOutputActor's send() behaviour to blocking. It
 * means the Send() method will block the execution when mailbox is full.
 *
 * @author Zdenko Vrabel (vrabel.zdenko@gmail.com)
 */
public class FifoBlockingMailbox extends FifoMailbox
{

	/**
	 * Constructor
	 */
	public FifoBlockingMailbox(int queueCapacity)
	{
		super(queueCapacity);
	}


	/**
	 * Blocking version of offer method
	 *
	 * @param msg
	 * @return
	 */
	@Override
	public boolean offer(Message<?> msg)
	{
		return offer(msg, -1);
	}
}
