/*****************************************************************************
 * Copyright 2012 Zdenko Vrabel
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package org.zdevra.sparrow.mailbox;

import org.zdevra.sparrow.message.Message;

/**
 * Interface describe mailbox for {@link org.zdevra.sparrow.actor.InputOutputActor actors}.
 * @author Zdenko Vrabel (vrabel.zdenko@gmail.com)
 */
public interface Mailbox
{
	/**
	 * Take and remove the message from mailbox if some message is available in mailbox. If not, method waits until
	 * a new message comes into queue or it will interrupt the current thread.
	 *
	 * @return pulled message
	 */
	public Message<?> take();


	/**
	 * Push the message into mailbox and returns true. If there is no space available or another problem occurs then
	 * method returns false.
	 *
	 * <p> Depending on mailbox implementation if the method is non-blocking or is blocking.
	 *
	 * @param msg
	 *          message which will be put into mailbox
	 *
	 * @return
	 */
	public boolean offer(Message<?> msg);


	/**
	 * Push the message into mailbox and returns true. If there is no space available or another problem occurs then
	 * method returns false. This method should block the exception until timeout expires.
	 *
	 * @param msg
	 *          message which will be put into mailbox
	 * @param timeout
	 *          timeout in miliseconds
	 * @return
	 */
	public boolean offer(Message<?> msg, long timeout);


}
