/*****************************************************************************
 * Copyright 2012 Zdenko Vrabel
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package org.zdevra.sparrow.smartproxy;

import org.zdevra.sparrow.channel.ChannelReference;
import org.zdevra.sparrow.channel.NamedChannelReference;
import org.zdevra.sparrow.actor.ActorBuilder;
import org.zdevra.sparrow.dsl.SparrowDsl;

/**
 * Main implementation of {@link SmartProxyBuilder builder}
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public class BaseSmartProxyBuilder implements SmartProxyBuilder
{
	//-------------------------------------------------------------------------------------------------------------------
	// members
	//-------------------------------------------------------------------------------------------------------------------

	/** dsl */
	protected final SparrowDsl dsl;

	/** smart proxy name, actor names are derived from this name */
	protected final String name;

	/** name of input channel from where requests comes */
	protected ChannelReference requestInputChannel;

	/** name of output channel where are requests forwarded */
	protected ChannelReference requestOutputChannel;

	/** name of input channel from where are replies received */
	protected ChannelReference replyInputChannel;

	//-------------------------------------------------------------------------------------------------------------------
	// constructor
	//-------------------------------------------------------------------------------------------------------------------


	public BaseSmartProxyBuilder(String name, SparrowDsl dsl)
	{
		this.name = name;
		this.dsl = dsl;
	}


	//-------------------------------------------------------------------------------------------------------------------
	// methods
	//-------------------------------------------------------------------------------------------------------------------


	/**
	 * {@inheritDoc}
	 */
	@Override
	public final SmartProxyBuilder requestsInput(ChannelReference channelName)
	{
		this.requestInputChannel = channelName;
		return this;
	}


	/**
	 * {@inheritDoc}
	 */
	@Override
	public final SmartProxyBuilder requestsOutput(ChannelReference channelName)
	{
		this.requestOutputChannel = channelName;
		return this;
	}


	/**
	 * {@inheritDoc}
	 */
	@Override
	public final SmartProxyBuilder repliesInput(ChannelReference channelName)
	{
		this.replyInputChannel = channelName;
		return this;
	}


	/**
	 * {@inheritDoc}
	 */
	@Override
	public void build()
	{
		SmartProxy proxy = new SmartProxy();

		dsl.from( requestInputChannel )
				  .actor( proxy.getRequestActor() )
				  .namedAs(this.name + "-request")
				  .to(requestOutputChannel);

		dsl.from( replyInputChannel )
				  .actor( proxy.getReplyActor() )
				  .namedAs(this.name + "-reply");
	}
}
