package org.zdevra.sparrow.splitter;

import org.zdevra.sparrow.message.Message;
import org.zdevra.sparrow.message.MessageBuilder;
import org.zdevra.sparrow.actor.OutputActorHandler;

import java.util.List;

/**
 * Actor handler follows Splitter integration pattern which split the message into more messages in sequence
 * that sequence is identified by correlation ID.
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
class SplitterActorHandler extends OutputActorHandler<Object>
{
	//-------------------------------------------------------------------------------------------------------------------
	// members
	//-------------------------------------------------------------------------------------------------------------------

	/** split strategy how message will be split */
	private final SplitterStrategy strategy;


	//-------------------------------------------------------------------------------------------------------------------
	// constructors & methods
	//-------------------------------------------------------------------------------------------------------------------

	/**
	 * Constructor
	 *
	 * @param strategy
	 *           strategy determines how the message will be split
	 */
	SplitterActorHandler(SplitterStrategy strategy)
	{
		this.strategy = strategy;
	}


	/**
	 * {@inheritDoc}
	 */
	@Override
	public final void act(Message<Object> msg)
	{
		List<Message<?>> sequence = strategy.split(msg);
		if (sequence == null) {
			return;
		}

		for (int i = 0; i < sequence.size(); ++i)
		{
			Message<?> seqMsg = sequence.get(i);
			Message<?> msgToSend =
					  MessageBuilder.fromMessage(seqMsg)
								 .correlationIdForMessage(msg)
								 .sequenceNumber(i, sequence.size())
								 .copyHeaders(seqMsg.getHeader())
								 .build();

			output().send(msgToSend);
		}
	}
}
