package org.zdevra.sparrow.splitter;

import org.zdevra.sparrow.actor.ActorHandler;

/**
 * Basic implementation of {@link SplitterBuilder} that is used primary inside
 * {@link org.zdevra.sparrow.AbstractSparrowEngine AbstractSparrowEngine}
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public class SplitterBuilderImpl implements SplitterBuilder
{
	//-------------------------------------------------------------------------------------------------------------------
	// members
	//-------------------------------------------------------------------------------------------------------------------

	/** splitter strategy for which the builder builds actor handler */
	protected SplitterStrategy strategy;

	//-------------------------------------------------------------------------------------------------------------------
	// methods
	//-------------------------------------------------------------------------------------------------------------------

	/**
	 * {@inheritDoc}
	 */
	@Override
	public SplitterBuilder ofInstance(SplitterStrategy strategy)
	{
		this.strategy = strategy;
		return this;
	}


	@Override
	public SplitterBuilder of(Class<?> strategyClass)
	{
		try {
			if (SplitterStrategy.class.isAssignableFrom(strategyClass)) {
				this.strategy = (SplitterStrategy) strategyClass.newInstance();
			}
		} catch (InstantiationException e) {
			//todo: handle the exception
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			//todo: handle the exception
			e.printStackTrace();
		}

		return this;
	}


	/**
	 * {@inheritDoc}
	 */
	@Override
	public ActorHandler<?> build()
	{
		return new SplitterActorHandler(this.strategy);
	}
}
